@@ -7,6 +7,8 @@
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.
 
+from itertools import chain
+
 import jsonpickle.util as util
 import jsonpickle.tags as tags
 import jsonpickle.handlers as handlers
@@ -199,14 +201,13 @@ class Pickler(object):
         # Support objects with __getstate__(); this ensures that
         # both __setstate__() and __getstate__() are implemented
         has_getstate = hasattr(obj, '__getstate__')
-        has_getstate_support = has_getstate and hasattr(obj, '__setstate__')
 
         if has_class and not util.is_module(obj):
             module, name = _getclassdetail(obj)
             if self.unpicklable:
                 data[tags.OBJECT] = '%s.%s' % (module, name)
             # Check for a custom handler
-            handler = handlers.get(type(obj))
+            handler = handlers.get(obj.__class__)
             if handler is not None:
                 return handler(self).flatten(obj, data)
 
@@ -220,7 +221,7 @@ class Pickler(object):
 
         if util.is_dictionary_subclass(obj):
             self._flatten_dict_obj(obj, data)
-            if has_getstate_support:
+            if has_getstate:
                 self._getstate(obj, data)
             return data
 
@@ -229,7 +230,7 @@ class Pickler(object):
             if util.is_sequence_subclass(obj):
                 return self._flatten_sequence_obj(obj, data)
 
-            if has_getstate_support:
+            if has_getstate:
                 return self._getstate(obj, data)
 
             # hack for zope persistent objects; this unghostifies the object
@@ -257,15 +258,31 @@ class Pickler(object):
 
         # the collections.defaultdict protocol
         if hasattr(obj, 'default_factory') and callable(obj.default_factory):
-            flatten('default_factory', obj.default_factory, data)
+            factory = obj.default_factory
+            if util.is_type(factory):
+                # Reference the type
+                value = _mktyperef(factory)
+            else:
+                # Create an instance from the factory and assume that the
+                # resulting instance is a suitable examplar.
+                value = self._flatten(handlers.CloneFactory(factory()))
+            data['default_factory'] = value
 
         return data
 
     def _flatten_newstyle_with_slots(self, obj, data):
         """Return a json-friendly dict for new-style objects with __slots__.
         """
-        for k in obj.__slots__:
-            self._flatten_key_value_pair(k, getattr(obj, k), data)
+        allslots = [getattr(cls, '__slots__', tuple())
+                        for cls in type(obj).mro()]
+        flatten = self._flatten_key_value_pair
+        for k in chain(*allslots):
+            try:
+                value = getattr(obj, k)
+            except AttributeError:
+                # The attribute may have been deleted
+                continue
+            flatten(k, value, data)
         return data
 
     def _flatten_key_value_pair(self, k, v, data):
