@@ -31,9 +31,6 @@ def _make_backend(backend):
     else:
         return backend
 
-def _supports_getstate(obj, instance):
-    return hasattr(instance, '__setstate__') and has_tag(obj, tags.STATE)
-
 
 class Unpickler(object):
 
@@ -128,8 +125,16 @@ class Unpickler(object):
         else:
             return self._restore_object_instance(obj, cls)
 
+    def _loadfactory(self, obj):
+        try:
+            default_factory = obj['default_factory']
+        except KeyError:
+            return None
+        del obj['default_factory']
+        return self._restore(default_factory)
+
     def _restore_object_instance(self, obj, cls):
-        factory = loadfactory(obj)
+        factory = self._loadfactory(obj)
         args = getargs(obj)
         if args:
             args = self._restore(args)
@@ -155,11 +160,13 @@ class Unpickler(object):
         return self._restore_object_instance_variables(obj, instance)
 
     def _restore_object_instance_variables(self, obj, instance):
+        restore_key = self._restore_key_fn()
         for k, v in sorted(obj.items(), key=util.itemgetter):
             # ignore the reserved attribute
             if k in tags.RESERVED:
                 continue
             self._namestack.append(k)
+            k = restore_key(k)
             # step into the namespace
             value = self._restore(v)
             if (util.is_noncomplex(instance) or
@@ -179,14 +186,20 @@ class Unpickler(object):
                 for v in obj[tags.SEQ]:
                     instance.add(self._restore(v))
 
-        if _supports_getstate(obj, instance):
-            self._restore_state(obj, instance)
+        if has_tag(obj, tags.STATE):
+            instance = self._restore_state(obj, instance)
 
         return instance
 
     def _restore_state(self, obj, instance):
-        state = self._restore(obj[tags.STATE])
-        instance.__setstate__(state)
+        if hasattr(instance, '__setstate__'):
+            state = self._restore(obj[tags.STATE])
+            instance.__setstate__(state)
+        else:
+            # __setstate__ is not implemented so that means that the best
+            # we can do is return the result of __getstate__() rather than
+            # return an empty shell of an object.
+            instance = self._restore(obj[tags.STATE])
         return instance
 
     def _restore_list(self, obj):
@@ -204,16 +217,37 @@ class Unpickler(object):
 
     def _restore_dict(self, obj):
         data = {}
+        restore_key = self._restore_key_fn()
         for k, v in sorted(obj.items(), key=util.itemgetter):
             self._namestack.append(k)
-            if self.keys and k.startswith(tags.JSON_KEY):
-                k = decode(k[len(tags.JSON_KEY):],
-                           backend=self.backend, context=self,
-                           keys=True, reset=False)
+            k = restore_key(k)
             data[k] = self._restore(v)
             self._namestack.pop()
         return data
 
+    def _restore_key_fn(self):
+        """Return a callable that restores keys
+
+        This function is responsible for restoring non-string keys
+        when we are decoding with `keys=True`.
+
+        """
+        # This function is called before entering a tight loop
+        # where the returned function will be called.
+        # We return a specific function after checking self.keys
+        # instead of doing so in the body of the function to
+        # avoid conditional branching inside a tight loop.
+        if self.keys:
+            def restore_key(key):
+                if key.startswith(tags.JSON_KEY):
+                    key = decode(key[len(tags.JSON_KEY):],
+                                 backend=self.backend, context=self,
+                                 keys=True, reset=False)
+                return key
+        else:
+            restore_key = lambda key: key
+        return restore_key
+
     def _refname(self):
         """Calculates the name of the current location in the JSON stack.
 
@@ -286,24 +320,6 @@ def loadclass(module_and_name):
         return None
 
 
-def loadfactory(obj):
-    try:
-        default_factory = obj['default_factory']
-    except KeyError:
-        return None
-    try:
-        type_tag = default_factory[tags.TYPE]
-    except:
-        return None
-
-    typeref = loadclass(type_tag)
-    if typeref:
-        del obj['default_factory']
-        return typeref
-
-    return None
-
-
 def getargs(obj):
     try:
         seq_list = obj[tags.SEQ]
